#include "lasp_config.h"
#include <pybind11/pybind11.h>

/*! \mainpage
 *
 * \section intro_sec Introduction
 *
 * Welcome to the LASP (Library for Acoustic Signal Processing) code
 * documentation. The code comprises a part which is written in C++, a part
 * that is written in Python, and a part that functions as glue, which is
 * Pybind11 C++ glue code. An example of such a file is lasp_cpp.cpp.
 * This is the internal documentation of LASP. It serves as background
 * information for programmers.
 *
 * \section Installation
 *
 * For the installation manual, please refer to the <a
 * href="https://code.ascee.nl/ASCEE/lasp/src/branch/master/README.md">README</a>
 * of the Git repository.
 *
 *
 * \section Usage
 *
 * Some usage examples are given in the <a href=
 * "https://code.ascee.nl/ASCEE/lasp/src/branch/master/examples">examples</a>
 * directory of the repository.
 *
 * */

/**
 * \defgroup pybind Pybind11 wrapper code
 * @{
 *
 */
namespace py = pybind11;

void init_dsp(py::module &m);
void init_deviceinfo(py::module &m);
void init_daqconfiguration(py::module &m);
void init_daq(py::module &m);
void init_streammgr(py::module &m);
void init_datahandler(py::module &m);
void init_siggen(py::module &m);

PYBIND11_MODULE(lasp_cpp, m) {

  init_dsp(m);
  init_deviceinfo(m);
  init_daqconfiguration(m);
  init_daq(m);
  init_streammgr(m);
  init_datahandler(m);
  init_siggen(m);

}
/** @} */
